classdef HelperImageDataReader < matlab.System
    % Read image data in Simulink.
    
    % This is a helper class and may be removed or modified in a future
    % release.
    
    % Copyright 2019 The MathWorks, Inc.
    properties (Nontunable)
        % TimeStep Time step between two image scans
        TimeStep = 0.1;
    end
    
    properties (Nontunable)
        % ImageSize Size of the image
        ImageSize = [144 192 3];
    end
    
    properties (Access = protected)
        Counter = 0;
    end
    
    properties (Access = protected)
        pImageData;
    end
    
    properties (Nontunable)
        %ImageFile Name of the Image file
        ImageFile = 'imageData_35seconds';
        
        % ImageDataVariableName Name of the image data in MAT file
        ImageDataVariableName = 'allImageData';
    end
    
    methods
        function obj = HelperImageDataReader(varargin)
            setProperties(obj,nargin,varargin{:});
        end
    end

    methods (Access = protected)
        function setupImpl(obj)
            s = load([obj.ImageFile,'.mat'],obj.ImageDataVariableName);
            imageData = s.(obj.ImageDataVariableName);            
            obj.pImageData = imageData;
        end
        
        function imageData = stepImpl(obj)
            time = getCurrentTime(obj);
            sts = getSampleTime(obj);
            dT = sts.SampleTime;
            counter = max(1,floor(time/dT));
            imageData = obj.pImageData{counter};
        end
        
        function out = getOutputNamesImpl(~)
           out = "Image"; 
        end
        
        function out1 = getOutputSizeImpl(obj)
            out1 = obj.ImageSize;
        end
                    
        function d1 = getOutputDataTypeImpl(~)
           d1 = "uint8";
        end
        
        function varargout = isOutputComplexImpl(~)
            varargout = cell(nargout,1);
            for i = 1:nargout
                varargout{i} = false;
            end
        end
        
        function varargout = isOutputFixedSizeImpl(~)
            varargout = cell(nargout,1);
            for i = 1:nargout
                varargout{i} = true;
            end
        end

        function sts = getSampleTimeImpl(obj)
            % Define sample time type and parameters
            sts = obj.createSampleTime("Type", "Discrete", "SampleTime", obj.TimeStep);

            % Example: specify discrete sample time
            % sts = obj.createSampleTime("Type", "Discrete", ...
            %     "SampleTime", 1);
        end
        
        function icon = getIconImpl(~)
            icon = ["Helper","Image Data Reader"];
        end
        
    end

    methods(Access = protected, Static)
        function header = getHeaderImpl
            header = matlab.system.display.Header('Title','HelperLidarDataReader',...
                'Text',HelperImageDataReader.descriptionText);
        end
        
        function simMode = getSimulateUsingImpl
            % Return only allowed simulation mode in System block dialog
            simMode = "Interpreted execution";
        end

        function flag = showSimulateUsingImpl
            % Return false if simulation mode hidden in System block dialog
            flag = false;
        end
        
        function text = descriptionText
           text = sprintf(['A helper block to read image data from a MAT file. ',...
               'The MAT file stores image data as a cell array of images. ',...
               'At each step, the block gets the current time from Simulink and uses the time-step between two image readings to ',...
               'compute the value of current image.']);
        end 
    end
end