classdef AEBLid_ObjectDetection_Var_C_V01 < matlab.System
    % --> Nutzung Polygon zur Auswahl AEB-Reaktion für geringeste Distanz
%% Input (Code is running fast but box detection needs time to check all boxes) 40fps->10fps, configure Lidar 1024x20  
    properties        
        steeringFactor = 470/36; %Factor for Steering (Trajectory curving) SteeringWheelMax/WeehlAngleMax in deg
        lanewidth = 2.1; %Lanewidth in m
        wheelbase = 2.565;  %Distance between front and rear axel in m
        hoodOffset = 2.565/3; %Offset Sensor/Front in m
        simulationtime = 2.5;  %SimTime Trajectory in seconds (set ration between brake and attention zone) max 3.6 -> look function detect objects and change speed ratio
        dt = 0.25;  %Trajectory smoothening in seconds
        BoundingBox = [0.1 0.8 0.1 0.8 0.6 2.5]; %BoundingBoxSize Min,Max per Axis (x,y,z)
        p1_x = [0.0 1.5 1.5 0.0 -1.5 -1.5 0];
        p1_y = [0.0 0.0 8.0 10.0 8.0 0.0 0.0];
    end
   
    methods 
        function obj = AEBLid_ObjectDetection_Var_C_V01(varargin)
            setProperties(obj,nargin,varargin{:})
        end
    end
    

    methods (Access = protected)

        function [emergency_brake,warning,fps] = stepImpl(obj,pc_in,steering) 
%% Init
            % persistent player
            % 
            % if isempty(player)
            %     player=pcplayer([-10 10], [-5 20], [-4 3]); %see indices roi
            % 
            % end
            emergency_brake = 0;
            warning = 0;
            fps = 0;
            AEB_th = [12 7 4 2]; % Threshold for warning/braking
            farbe = 'kgymr';
            p2_x = obj.p1_x*0.8;
            p2_y = obj.p1_y*0.9;
            p3_x = obj.p1_x*0.6;
            p3_y = obj.p1_y*0.8;
            p4_x = obj.p1_x*0.4;
            p4_y = obj.p1_y*0.5;
                     
tic
                try      
%% PT-Cloud read/disp                    
 
                    ptCloud = pointCloud(pc_in);
                    
                    %___ for simulation only !!! ___
                    ptCloud = pctransform(ptCloud,rigidtform3d([0 0 90],[0 -3 0]));
                    indices = findPointsInROI(ptCloud,[-2.5 2.5 0 20 -3 2]);
                    ptCloud = select(ptCloud,indices);
                    

%% Detect Objects

	                referenceVector = [0 0 1];
                    steering = steering/obj.steeringFactor; %wheelangle
 

	                [~, ~, outliers] = pcfitplane(ptCloud,0.3,referenceVector);

	                ptCloudWithoutGround = select(ptCloud,outliers,'OutputSize','full');
	                
                    % player.view(ptCloudWithoutGround);
                    % player.view(ptCloud);

	                [labels,numClusters] = pcsegdist(ptCloudWithoutGround,0.1);
                    % [labels,numClusters] = pcsegdist(ptCloud,0.2);

                    % F1 = figure(1,"Position",[357.6667    8.3333  470.6667  809.3333]); cla;
                    F1 = figure(1); cla;
                    % AEB_temp = zeros(numClusters,1);
                    for i = 1:1:numClusters
                        idxValidPoints = find(labels == i);
                        labelColorIndex = labels(idxValidPoints);
                        segmentedPtCloud = select(ptCloudWithoutGround,idxValidPoints);
                        
                        dist = sqrt(segmentedPtCloud.Location(:,1).^2 + segmentedPtCloud.Location(:,2).^2);
                        [dist_min, idx] = min(dist);
                        dist_min_all(i,1:4) = [segmentedPtCloud.Location(idx,1), segmentedPtCloud.Location(idx,2), segmentedPtCloud.Location(idx,3), dist_min];
                    end


                    line(dist_min_all(:,1),dist_min_all(:,2),dist_min_all(:,3),'Marker','*','LineStyle','none'); grid on;
                    line(obj.p1_x,obj.p1_y,'Color','g','LineWidth',2);
                    line(p2_x,p2_y,'Color','y','LineWidth',2);
                    line(p3_x,p3_y,'Color','m','LineWidth',2);
                    line(p4_x,p4_y,'Color','r','LineWidth',2);


                    xlim([-2.5 2.5]); ylim([0 12])
                    [dist_min, idx2] = min(dist_min_all(:,4));
                    obj_x = min(dist_min_all(idx2,1))
                    obj_y = min(dist_min_all(idx2,2))
                    % line(obj_x,obj_y,'Marker','o','MarkerEdgeColor',[0.5 0.5 0.5],'MarkerFaceColor',[0.5 0.5 0.5],'MarkerSize',15);
                    
                    AEB_temp = 0;
                    [in1, on1] = inpolygon(obj_x,obj_y,obj.p1_x,obj.p1_y);
                    if in1
                        AEB_temp = 1;
                        [in2, on2] = inpolygon(obj_x,obj_y,p2_x,p2_y);
                        if in2
                            AEB_temp = 2;
                            [in3, on3] = inpolygon(obj_x,obj_y,p3_x,p3_y);
                            if in3
                                AEB_temp = 3;
                                [in4, on4] = inpolygon(obj_x,obj_y,p4_x,p4_y);
                                if in4
                                    AEB_temp = 4;
                                end                            
                            end
                        end
                    end
                    line(obj_x,obj_y,'Marker','o','MarkerEdgeColor',farbe(AEB_temp+1),'MarkerFaceColor',farbe(AEB_temp+1),'MarkerSize',15);

                    if AEB_temp
                        emergency_brake = AEB_temp;
                        % line(obj_x,obj_y,'Marker','o','MarkerEdgeColor',farbe(emergency_brake),'MarkerFaceColor',farbe(emergency_brake),'MarkerSize',15);
                        % xlabel('lateral distance / m'); ylabel('longitudinal distance / m'); 
                        warning = 1;
                    else
                        emergency_brake = 0;
                        warning = 0;
                    end

                    toc      
                    fps=1/toc;

                catch
                    disp("Detection Failed!");
                end             
        end

        
%% Output
        function [o1,o2,o3] = getOutputSizeImpl(obj)
            o1 = [1 1];
            o2 = [1 1];
            o3 = [1 1];
        end

        function [o1,o2,o3] = getOutputDataTypeImpl(obj)
            % Return data type for output port to be same as input port
            inType = propagatedInputDataType(obj, 1);
            o1 = inType;
            o2 = inType;
            o3 = inType;
        end

        function [o1,o2,o3] = isOutputComplexImpl(~)
            % Return output port complexity to be real
            o1 = false;
            o2 = false;
            o3 = false;
        end

        function [o1,o2,o3] = isOutputFixedSizeImpl(~)
            % Return true for each output port with fixed size
            o1 = true;
            o2 = true;
            o3 = true;
        end

        % Anpassung für Akzeptanz variable Eingangssignale
        function flag = isInputSizeMutableImpl(~, ~)
         flag = true;
        end

    end

%% Setup
    methods(Static,Access = protected)
        function simMode = getSimulateUsingImpl
            simMode = 'Interpreted execution';
        end

        function sts = getSampleTimeImpl(obj)
            sts = createSampleTime(obj, 'Type', 'Discrete', 'SampleTime', 0.2);
        end

    end

end
    

